@@ -1,5 +1,7 @@
-;; Incremental search for SKK (version 1.12 of October 14, 1992)
+;; Incremental search for SKK (version 1.14 of May 17, 1994)
 
+;; version 1.14 released on May 17, 1994
+;; version 1.13 released on April 13, 1994
 ;; version 1.12 released on October 14, 1992
 ;; version 1.11 released on April 13, 1992
 ;; version 1.10 released on March 26, 1992
@@ -99,16 +101,21 @@ search string.")
   "*If non-nil, it is inserted between an ASCII character and a space
 charcter.  The space character will be deleted.")
 
+;; modified by Tsugutomo Enami, 1994.4.18
+
 (defun j-eval-one-command (char)
   "Interpret CHAR under the current keymaps getting additional keys from
 the keyboard if CHAR does not correspond to a command."
-  (let ((com (j-get-editor-command (current-local-map) global-map char)))
-    (condition-case conditions
-	(command-execute com)
-      (quit
-       (j-message-conditions conditions))
-      (error
-       (j-message-conditions conditions)))))
+  (setq unread-command-char char)
+  (condition-case conditions
+      (let ((keys (read-key-sequence nil)))
+	(setq last-command-char (aref keys (1- (length keys))))
+	(command-execute (key-binding keys)))
+    (quit
+     (j-message-conditions conditions))
+    (error
+     (j-message-conditions conditions))))
+
 
 (defun j-message-conditions (conditions)
   (let ((case (car conditions)) (msg (cdr conditions)))
