@@ -1,6 +1,6 @@
 /* skkserv.c
- * SKK Server Version 3.7.5 of May 13, 1992
- * Copyright (C) 1992 Yukiyoshi Kameyama (kam@riec.tohoku.ac.jp)
+ * SKK Server Version 3.9.2 of Jun 7, 1994
+ * Copyright (C) 1994 Yukiyoshi Kameyama (kam@riec.tohoku.ac.jp)
  *
  * Programmed by
  * Yukiyoshi Kameyama (kam@riec.tohoku.ac.jp)
@@ -12,6 +12,7 @@
  * Chikahiro Kimura (kimura@2oa.kb.nec.co.jp) for "ntohs" bug
  * Koji Kawaguchi (guchi@pfu.fujitsu.co.jp) for various improvement
  * Hironobu Takahashi (takahasi@tiny.or.jp) for patch for char/int typing
+ * Kazushi Marukawa (kazushi@kubota.co.jp) for cleaning up behavior in PRIVATE mode
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -105,9 +106,10 @@
  *	Global Variables
  */
 
-char	pgmver[] = "3.7.5 ";	/* version number */
+char	pgmver[] = "3.9.2 ";	/* version number */
 
 char	*pgmnm;		/* program name */
+char	*jname;		/* name of shared dictionary */
 char	hname[BUFSIZ];	/* hostname and IP address */
 FILE	*jisho;		/* SKK jisho */
 FILE	*errout;	/* Error Output */
@@ -126,14 +128,24 @@ char *argv[];
 {
   int	parg;
   int	setjisho = 0;
-  char	*jname;		/* name of shared dictionary */
   int	ctlterm;	/* fildes for control terminal */
+  void	reread();
 
   pgmnm = argv[0];
   debug = 0;
   errout = stderr;
   portnum = 0;
 
+  /* The following patch (a work-around due to "rsh" bug) was pointed 	*/
+  /* out by kazushi@kubota.co.jp in the message "714" in SKK-ML		*/
+  /* Not knowing the reason in detail, I added it in any way, since	*/
+  /* it does not have any bad effects even if not useful.		*/
+  {	
+    int i;
+    for (i = 3; i < 15; i++)
+      (void) close(i);
+  }
+    
   for (parg = 1; parg < argc; ) {
     if (*argv[parg] == '-') {
       switch (*(argv[parg]+1)) {
@@ -172,6 +184,12 @@ char *argv[];
 
   set_hname();
 
+  /* make socket */
+  mksock();
+
+  /* make table of contents for jisho */
+  mkjtab();
+
   if (!debug) {
     /* parent process exits now */
     if (fork() != 0) exit(0);
@@ -203,11 +221,8 @@ char *argv[];
     if (errout != stderr) fclose(stderr);
   }
 
-  /* make socket */
-  mksock();
-
-  /* make table of contents for jisho */
-  mkjtab();
+  signal(SIGINT, reread);
+  /* 1993/6/5 by kam, re-read dictionary by "INT" signal */
 
   nclients = 0;
   main_loop();
@@ -229,6 +244,7 @@ mksock()
 {
   struct sockaddr_in	sin;
   struct servent	*sp;
+  int	optbuf = 1; /* enable socket REUSEADDR */
   
 #ifdef NO_BZERO
   memset((char*)&sin, 0, sizeof(sin));
@@ -250,6 +266,9 @@ mksock()
 
   if ((initsock = socket(PF_INET, SOCK_STREAM, 0)) < 0)
     err("socket error; socket cannot be created");
+  if (setsockopt(initsock, SOL_SOCKET, SO_REUSEADDR, 
+		 &optbuf, sizeof(optbuf)) < 0)
+    err("socket error; cannot set socket option");
   if (bind(initsock, (struct sockaddr *)&sin, sizeof(sin))< 0) 
     err("bind error; the socket is already used");
   if (listen(initsock, MAXQUE) < 0) 
@@ -287,6 +306,22 @@ mkjtab()
     mkoldjtab(buf);
 }
 
+/* 1993/6/5 by kam, re-read dictionary by "INT" signal */
+void reread()
+{
+  if (fclose(jisho) < 0) {
+    fprintf(stderr, 
+	    "%s: closing shared dictionary \"%s\" failed\n", pgmnm, jname);
+    exit(1);
+  }
+  if ((jisho = fopen(jname, "r")) == NULL) {
+    fprintf(stderr, 
+	    "%s: rereading shared dictionary \"%s\" failed\n", pgmnm, jname);
+    exit(1);
+  }
+  mkjtab();
+}
+
 mkoldjtab(s)
 unsigned char	*s;
 {
@@ -371,7 +406,7 @@ mknewjtab()
     }
   }
   if (target != STRMARK)
-    err("format error; new-style dictionary shold have two parts");
+    err("format error; new-style dictionary should have two parts");
   while (code >= KANA_START - 1) {
     *pjtab++ = ftell(jisho) - 2;
     code --;
@@ -432,8 +467,11 @@ main_loop()
   FD_ZERO(&exceptfds);
   for(;;) {	/* infinite loop; waiting for client's request */
     readfds = getrfds();
-    if (select(MAXDTAB, &readfds, &writefds, &exceptfds, NULL) < 0)
+    if (select(MAXDTAB, &readfds, &writefds, &exceptfds, NULL) < 0) {
+      if (errno == EINTR) /* if signal happens */
+	continue;
       err("select error; something wrong happened with the socket");
+    }
     if (debug) {
       fprintf(errout, "select: read file descriptor is %d\n", readfds);
       fflush(errout);
@@ -696,3 +734,4 @@ set_hname()
 #endif
   strcat(hname, ": ");
 }
+
